<?php

namespace EasyContent\WordPress;

use EasyContent\WordPress\ValueObjects\AttachmentPost;

final class Attachment
{
    /** @var AttachmentPost $attachmentPost */
    private $attachmentPost;

    /** @var false|string $extension */
    private $extension;

    /** @var false|string $mimeType */
    private $mimeType;

    private function __construct() { }

    /**
     * @param \WP_Post $post
     * @return Attachment
     * @throws \Exception
     */
    public static function fromWordPressPost(\WP_Post $post)
    {
        if ('attachment' !== $post->post_type) {
            throw new \Exception(__('Invalid post type', EASYCONTENT_TXTDOMAIN));
        }

        $filePath = get_attached_file($post->ID);

        return (new self)->create(new AttachmentPost($post->ID, $filePath));
    }

    /**
     * @param AttachmentPost $post
     * @return Attachment
     */
    public static function fromAttachmentPost(AttachmentPost $post)
    {
        return (new self)->create($post);
    }

    /**
     * @param AttachmentPost $post
     * @return $this
     */
    private function create(AttachmentPost $post)
    {
        $this->attachmentPost = $post;

        $data = wp_check_filetype($this->getFilePath());

        $this->extension = $data['ext'];
        $this->mimeType = $data['type'];

        return $this;
    }

    /**
     * @return AttachmentPost
     */
    public function getAttachmentPost()
    {
        return $this->attachmentPost;
    }

    public function getFilePath()
    {
        return $this->attachmentPost->getLocalUrl();
    }


    public static function updateMetaData($attachmentId, $filePath)
    {
        if ( ! function_exists( 'wp_generate_attachment_metadata' ) ) {
            require_once( ABSPATH . 'wp-admin/includes/image.php' );
        }

        $meta = wp_generate_attachment_metadata( $attachmentId, $filePath );
        wp_update_attachment_metadata( $attachmentId, $meta );
    }


    /**
     * @param string $url Image URL
     * @param int $postId
     * @return null|Attachment
     * @throws \Exception
     */
    public static function createFromUrl($url, $postId)
    {
        $response = wp_remote_get($url);
        $responseCode = wp_remote_retrieve_response_code($response);

        if (200 !== $responseCode) {
            return null;
        }

        if (!$responseBody = wp_remote_retrieve_body($response)) {
            return null;
        }

        $upload = wp_upload_bits( basename( $url ), null, $responseBody );

        if ( ! empty( $upload['error'] ) ) {
            return null;
        }

        self::resize( $upload['file'] );

        $filePath = $upload['file'];
        $fileName = basename( $filePath );
        $attachmentTitle = sanitize_file_name( pathinfo( $fileName, PATHINFO_FILENAME ) );

        $postInfo = [
            'guid'           => $upload['url'],
            'post_mime_type' => $upload['type'],
            'post_title'     => $attachmentTitle,
            'post_content'   => '',
            'post_status'    => 'inherit',
            'post_author'    => Plugin::getInstance()->getSettings()->getPostAuthorId(),
            'post_parent'    => $postId
        ];


        $attachmentId = wp_insert_attachment( $postInfo, $filePath, 0, true );

        if ( is_wp_error( $attachmentId ) ) {
            return null;
        }

        self::updateMetaData( $attachmentId, $filePath );

        update_post_meta( $attachmentId, 'ec-source-url', $url );

        try {
            $attachmentObject = self::fromWordPressPost(get_post($attachmentId));
        } catch ( \Exception $e ) {
            return null;
        }

        return $attachmentObject;
    }


    public function updateFromUrl($url)
    {
        $response = wp_remote_head($url);
        $responseCode = wp_remote_retrieve_response_code($response);

        if (204 !== $responseCode) {
            return false;
        }

        $attachmentSize = wp_remote_retrieve_header($response, 'Content-Length');

        if ( ! $attachmentSize ) {
            return false;
        }

        $attachmentSize = is_array($attachmentSize) ? (int)$attachmentSize[0] : (int)$attachmentSize;

        $attachmentFilePath = $this->getFilePath();
        $oldAttachmentSize = filesize( $attachmentFilePath );

        if ( $oldAttachmentSize === $attachmentSize ) {
            return true;
        }


        $response = wp_remote_get($url);
        $responseCode = wp_remote_retrieve_response_code($response);

        if (200 !== $responseCode) {
            return false;
        }

        if (!$responseBody = wp_remote_retrieve_body($response)) {
            return false;
        }

        $fh = fopen( $attachmentFilePath, 'w' );

        if ( false === $fh ) {
            return false;
        }

        $result = fwrite( $fh, $responseBody );
        fclose( $fh );

        if ( false === $result ) {
            return false;
        }

        self::resize( $attachmentFilePath );
        self::updateMetaData( $this->attachmentPost->getId(), $attachmentFilePath );

        return true;
    }


    public static function resize($filePath)
    {
        if ( ! is_file( $filePath ) ) {
            return false;
        }

        $wasModified = false;

        $extension = pathinfo( $filePath, PATHINFO_EXTENSION );
        $normalizedExtension = strtolower( $extension );

        $isJpg = 'jpg' === $normalizedExtension || 'jpeg' === $normalizedExtension;
        $isGif = 'gif' === $normalizedExtension;
        $isPng = 'png' === $normalizedExtension;

        if ( ! $isJpg && ! $isGif && ! $isPng ) {
            return false;
        }

        if ( self::isAnimatedGif( $filePath ) ) {
            return false;
        }

        $imageEditor = wp_get_image_editor( $filePath );

        if ( is_wp_error( $imageEditor ) ) {
            return false;
        }

        try {
            $settings = Plugin::getInstance()->getImageProcessingSettings();
        } catch ( \Exception $e ) {
            return false;
        }

        if ( $settings->resizingEnabled ) {
            $sizes = $imageEditor->get_size();

            if ( ( isset( $sizes['width'] ) && $sizes['width'] > $settings->maxWidth )
                || ( isset( $sizes['height'] ) && $sizes['height'] > $settings->maxHeight ) ) {

                $result = $imageEditor->resize( $settings->maxWidth, $settings->maxHeight, false );

                if ( false === $result || is_wp_error( $result ) ) {
                    return false;
                }

                $wasModified = true;
            }
        }


        // Regardless of resizing, image must be saved if recompressing

        if ( $isJpg ) {
            $result = $imageEditor->set_quality( $settings->jpgCompressionLevel );

            if ( ! is_wp_error( $result ) ) {
                $wasModified = true;
            }
        }


        // Only save image if it has been resized or need recompressing

        if ( $wasModified ) {
            $data = $imageEditor->save( $filePath );

            if ( is_wp_error( $data ) ) {
                return false;
            }
        }

        return true;
    }


    public static function isAnimatedGif($filePath)
    {
        if ( strtolower( pathinfo( $filePath, PATHINFO_EXTENSION ) ) !== 'gif' ) {
            return false;
        }

        $fh = fopen( $filePath, 'rb' );

        if ( false === $fh ) {
            return false;
        }

        $count = 0;
        //an animated gif contains multiple "frames", with each frame having a
        //header made up of:
        // * a static 4-byte sequence (\x00\x21\xF9\x04)
        // * 4 variable bytes
        // * a static 2-byte sequence (\x00\x2C) (some variants may use \x00\x21 ?)

        // We read through the file til we reach the end of the file, or we've found
        // at least 2 frame headers
        $chunk = false;

        while ( ! feof( $fh ) && $count < 2 ) {
            //add the last 20 characters from the previous string, to make sure the searched pattern is not split.
            $chunk = ( $chunk ? substr( $chunk, -20 ) : '' ) . fread( $fh, 1024 * 100 ); //read 100kb at a time
            $count += preg_match_all( '#\x00\x21\xF9\x04.{4}\x00(\x2C|\x21)#s', $chunk );
        }

        fclose( $fh );

        return $count > 1;
    }


    public function isJpg()
    {
        return $this->extension === 'jpg'
            || $this->extension === 'jpeg'
            || $this->mimeType  === 'image/jpg'
            || $this->mimeType  === 'image/jpeg';
    }


    public function isGif()
    {
        return $this->extension === 'gif' || $this->mimeType === 'image/gif';
    }


    public function isPng()
    {
        return $this->extension === 'png' || $this->mimeType === 'image/png';
    }


    public function isImage()
    {
        return $this->isJpg() || $this->isGif() || $this->isPng();
    }
}
