<?php

namespace EasyContent\WordPress\Implementation\Entity;

use EasyContent\WordPress\Attachment;
use EasyContent\WordPress\Domain\Seo\SeoPlugin;
use EasyContent\WordPress\Helpers\PostContentProcessor;
use EasyContent\WordPress\Plugin;
use EasyContent\WordPress\Source\Entity\Post;
use EasyContent\WordPress\Source\Entity\Post\PostStorage;
use EasyContent\WordPress\Source\Entity\Post\PostType;

final class DbPostStorage implements PostStorage
{
    public function getById($postId)
    {
        $data = self::getPostData($postId, SeoPlugin::fromPluginSettings());

        if (!$data) {
            throw new \InvalidArgumentException("Post {$postId} doesn't exist");
        }

        global $wpdb;
        $junctionTable = "{$wpdb->prefix}easycontent_article_post";

        $query = "SELECT `article_id` FROM `{$junctionTable}` WHERE `post_id` = %d LIMIT 1;";
        $query = $wpdb->prepare($query, $postId);

        $articleId = $wpdb->get_var($query);
        $articleId = $articleId !== null ? (int)$articleId : null;

        return Post::fromState($data, $articleId);
    }

    public function save(Post $post)
    {
        $hasKSES = false !== has_filter('content_save_pre', 'wp_filter_post_kses');

        if ($hasKSES) {
            kses_remove_filters();
        }

        $parameters = [
            'post_author'    => $post->getAuthorId(),
            'post_status'    => $post->getStatus(),
            'post_type'      => $post->getType(),
            'comment_status' => $post->getCommentStatus(),
            'post_title'     => $post->getTitle(),
            'post_excerpt'   => $post->getExcerpt(),
            'post_name'      => $post->getCustomSlug(),
            'post_content'   => '',
        ];

        if ($post->getId()) {
            $parameters['ID'] = $post->getId();
        }

        if ($post->isPost()) {
            if ($post->getCategoriesIds()) {
                $parameters['post_category'] = $post->getCategoriesIds();
            }

            if ($post->getTagsIds()) {
                $parameters['tags_input'] = $post->getTagsIds();
            }
        }

        if ($post->getDate()) {
            $parameters['post_date'] = $post->getDate();
            $parameters['post_date_gmt'] = $post->getDate();
        }

        $postId = wp_insert_post($parameters, true);

        if (is_wp_error($postId) || 0 === $postId) {
            throw new \Exception($postId->get_error_message());
        }

        if (!$post->getId()) {
            $post->setId($postId);
        }

        // Set post content
        $postContent = $post->getContent()
            ? (new PostContentProcessor($post->getContent(), $post->getId()))->run()
            : '';

        $result = wp_update_post([
            'ID' => $post->getId(),
            'post_content' => $postContent,
        ], true);

        if (is_wp_error($result) || 0 === $result) {
            throw new \Exception($result->get_error_message());
        }

        // TODO: Maybe need to update content inside $post object somehow
        // $this->_wpPostObject->post_content = $postContent;

        // Set featured image
        if ($post->getFeaturedImage()) {
            if ($attachment = Attachment::createFromUrl($post->getFeaturedImage()->getUrl(), $post->getId())) {
                set_post_thumbnail($post->getId(), $attachment->getAttachmentPost()->getId());

                if ($post->getFeaturedImage()->getAlt() !== '') {
                    update_post_meta(
                        $attachment->getAttachmentPost()->getId(),
                        '_wp_attachment_image_alt',
                        $post->getFeaturedImage()->getAlt()
                    );
                }
            }
        }

        // Update meta data
        if ($seoPlugin = SeoPlugin::fromPluginSettings()) {
            $seoPlugin->saveMetaTitle($post->getId(), $post->getMetaTitle());
            $seoPlugin->saveMetaDescription($post->getId(), $post->getMetaDescription());
        }

        if ($hasKSES) {
            kses_init_filters();
        }

        // TODO
        // wp_set_post_tags( $postId, $article->tags );

        global $wpdb;
        $junctionTable = "{$wpdb->prefix}easycontent_article_post";

        if ($post->getRelatedArticleId()) {
            // TODO
        }
    }

    /**
     * @param int $postId
     * @param null|SeoPlugin $seoPlugin
     * @return array|null
     */
    public static function getPostData($postId, $seoPlugin)
    {
        /** @var null|array $post */
        if (!$post = get_post($postId, ARRAY_A)) {
            return null;
        }

        if ($post['post_type'] !== PostType::POST && $post['post_type'] !== PostType::PAGE) {
            return null;
        }

        $post['ID'] = $postId;
        $post['post_author'] = (int)$post['post_author'];

        // Fetch categories ids
        $categories = wp_get_post_categories($postId);
        $post['categories_ids'] = is_wp_error($categories) ? [] : $categories;

        // Fetch tags ids
        $tags = wp_get_post_tags($postId, ['fields' => 'ids']);
        $post['tags'] = is_wp_error($tags) ? [] : $tags;

        // Fetch featured image data
        $url = get_the_post_thumbnail_url($postId, 'full') ?: '';
        $alt = get_the_post_thumbnail_caption($postId);

        $post['featured_image'] = $url !== false || $alt ? [
            'url' => $url ?: '',
            'alt' => $alt,
        ] : null;

        // Fetch meta data
        $post['meta_title'] = $seoPlugin ? $seoPlugin->getMetaTitle($postId) : '';
        $post['meta_description'] = $seoPlugin ? $seoPlugin->getMetaDescription($postId) : '';

        return $post;
    }
}
