<?php

namespace EasyContent\WordPress\Implementation\Entity;

use EasyContent\WordPress\Domain\Seo\SeoPlugin;
use EasyContent\WordPress\Implementation\Http\Client;
use EasyContent\WordPress\Implementation\Http\Exception\NotFoundException;
use EasyContent\WordPress\Implementation\Http\Method;
use EasyContent\WordPress\Source\Api\Entity\ImportedArticle;
use EasyContent\WordPress\Source\Api\Entity\Responses\ImportResponse;
use EasyContent\WordPress\Source\Entity\Article\ArticleId;
use EasyContent\WordPress\Source\Entity\EasyContentArticlesStorage;
use EasyContent\WordPress\Source\Exception\RemoteArticleNotFoundException;

final class RemoteArticlesStorage implements EasyContentArticlesStorage
{
    private $httpClient;

    public function __construct(Client $httpClient)
    {
        $this->httpClient = $httpClient;
    }

    public function findByIds(array $ids)
    {
        $query = http_build_query(['ids' => $ids]);
        $response = $this->httpClient->request(Method::get(), "articles?{$query}");
        $payload = json_decode($response->get_data(), true);

        if (!is_array($payload)) {
            return null;
        }

        global $wpdb;
        $articlesCacheTable = "{$wpdb->prefix}easycontent_article";
        $junctionTable = "{$wpdb->prefix}easycontent_article_post";

        $cachedArticleQuery = "SELECT * FROM `{$articlesCacheTable}` WHERE `id` = %d LIMIT 1;";
        $relationQuery = "SELECT `post_id` FROM `$junctionTable` WHERE `article_id` = %d LIMIT 1;";

        $seoPlugin = SeoPlugin::fromPluginSettings();

        foreach ($payload as $i => $article) {
            $result = $this->fetchLocalData($article['id'], $cachedArticleQuery, $relationQuery, $seoPlugin);

            $payload[$i]['cached'] = $result['cached'];
            $payload[$i]['related_post'] = $result['related_post'];
        }

        return ImportResponse::fromState($payload);
    }

    public function getById(ArticleId $id)
    {
        global $wpdb;
        $articlesCacheTable = "{$wpdb->prefix}easycontent_article";
        $junctionTable = "{$wpdb->prefix}easycontent_article_post";

        $cachedArticleQuery = "SELECT * FROM `{$articlesCacheTable}` WHERE `id` = %d LIMIT 1;";
        $relationQuery = "SELECT `post_id` FROM `$junctionTable` WHERE `article_id` = %d LIMIT 1;";

        $seoPlugin = SeoPlugin::fromPluginSettings();

        try {
            $response = $this->httpClient->request(Method::get(), "articles/{$id->asInt()}");
        } catch (NotFoundException $e) {
            throw RemoteArticleNotFoundException::fromException($e);
        }

        $article = json_decode($response->get_data(), true);

        $result = $this->fetchLocalData($article['id'], $cachedArticleQuery, $relationQuery, $seoPlugin);

        $article['cached'] = $result['cached'];
        $article['related_post'] = $result['related_post'];

        return ImportedArticle::fromState($article);
    }

    /**
     * @param int $articleId
     * @param string $cachedArticleQuery
     * @param string $relationQuery
     * @param null|SeoPlugin $seoPlugin
     * @return array
     */
    private function fetchLocalData($articleId, $cachedArticleQuery, $relationQuery, $seoPlugin)
    {
        global $wpdb;
        $result = [];

        $cached = $wpdb->get_row($wpdb->prepare($cachedArticleQuery, $articleId), ARRAY_A);
        $result['cached'] = $cached === null ? null : [
            'id' => (int)$cached['id'],
            'current_workflow_stage_id' => (int)$cached['current_workflow_stage_id'],
            'topic_name' => $cached['topic_name'],
            'title' => $cached['title'],
            'updated' => $cached['updated'] === null ? null : (int)$cached['updated'],
        ];
        $result['related_post'] = null;

        if (!$postId = $wpdb->get_var($wpdb->prepare($relationQuery, $articleId))) {
            return $result;
        }

        $result['related_post'] = DbPostStorage::getPostData((int)$postId, $seoPlugin);

        return $result;
    }

    public function updateStatus(ImportedArticle $article)
    {
        $uri = "articles/{$article->getId()}";
        $payload = ['workflow_stage_id' => $article->getCurrentWorkflowStatusId()];
        $this->httpClient->request(Method::patch(), $uri, $payload);
    }

    public function add(ImportedArticle $article)
    {
        $payload = $this->payloadFromArticle($article);
        $response = $this->httpClient->request(Method::post(), 'articles', $payload);
        $result = json_decode($response->get_data(), true);

        $article->setId((int)$result['id']);
    }

    public function update(ImportedArticle $article)
    {
        if ($article->getId() === null) {
            throw new \Exception("Can't update this article. Please add it to repository first");
        }

        $uri = "articles/{$article->getId()}/revisions";
        $payload = $this->payloadFromArticle($article);
        $this->httpClient->request(Method::post(), $uri, $payload);
    }

    /**
     * @param ImportedArticle $article
     * @return array
     */
    private function payloadFromArticle(ImportedArticle $article)
    {
        return [
            'title' => $article->getTitle(),
            'content' => $article->getContent(),
            'categories' => $article->getCategoriesIds(),
            'meta_title' => $article->getMetaTitle(),
            'meta_description' => $article->getMetaDescription(),
            'featured_image' => $article->getFeaturedImage() ? $article->getFeaturedImage()->getUrl() : '',
            'excerpt' => $article->getExcerpt(),
            // 'tags' => , // TODO: Array of strings or empty array
            'custom_post_slug' => $article->getCustomSlug(),
        ];
    }
}
