<?php

namespace EasyContent\WordPress\Implementation\Http;

use EasyContent\WordPress\Implementation\Entity\Misc\Referer;
use EasyContent\WordPress\Implementation\Http\Exception\NotFoundException;
use EasyContent\WordPress\Plugin;

final class Client
{
    const BASE_URI = EASYCONTENT_ENDPOINT . '/plugin/api/';

    /** @var string $key */
    private $key;
    /** @var null|Referer $referer */
    private $referer;

    /**
     * @param string $key
     * @param string|null $referer
     */
    public function __construct($key, $referer = null)
    {
        $this->key = $key;
        $this->referer = $referer;
    }

    public static function fromPluginSettings()
    {
        $plugin = Plugin::getInstance();

        return new self(
            $plugin->getSettings()->publicKey,
            Referer::fromServer()
        );
    }

    /**
     * @param Method $method
     * @param string $uri
     * @param array $payload
     * @return \WP_HTTP_Requests_Response
     * @throws \Exception
     */
    public function request(Method $method, $uri, array $payload = [])
    {
        $response = wp_remote_request(
            $this->buildUrl($uri),
            $this->buildArguments($method->asString(), $payload)
        );

        if (is_wp_error($response)) {
            throw new \Exception(implode(PHP_EOL, $response->errors));
        }

        /** @var null|\WP_HTTP_Requests_Response $responseObject */
        $responseObject = $response['http_response'];

        if (null === $responseObject) {
            throw new \Exception('Unknown error during HTTP request');
        }

        $statusCode = $responseObject->get_status();

        if ($statusCode === NotFoundException::CODE) {
            throw new NotFoundException;
        } elseif ($statusCode < 200 || $statusCode >= 300) {
            throw new \Exception($response['response']['message'], $statusCode);
        }

        return $responseObject;
    }

    /**
     * @param string $uri
     * @return string
     */
    private function buildUrl($uri)
    {
        return self::BASE_URI . $uri;
    }

    /**
     * @return string[]
     */
    private function buildHeaders()
    {
        $headers = [
            'Authorization' => "Bearer {$this->key}",
            'Content-Type' => 'application/json',
        ];

        if ($this->referer) {
            $headers['Referer'] = $this->referer->asString();
        }

        return $headers;
    }

    private function buildArguments($method, $body = null)
    {
        return [
            'method' => $method,
            'redirection' => 0,
            'headers' => $this->buildHeaders(),
            'body' => $body ? json_encode($body) : null,
        ];
    }
}
