<?php

namespace EasyContent\WordPress;

use EasyContent\WordPress\Db\Models\Article as ArticleDbModel;
use EasyContent\WordPress\Db\Models\ArticlePost;
use EasyContent\WordPress\Domain\Seo\SeoPlugin;
use EasyContent\WordPress\Helpers\PostContentProcessor;
use EasyContent\WordPress\Helpers\TimeHelper;
use EasyContent\WordPress\Source\UseCase\PushPost\Command as PushPostCommand;
use EasyContent\WordPress\Source\UseCase\PushPost\Handler as PushPostHandler;
use EasyContent\WordPress\Source\UseCase\PullPost\Command as PullPostCommand;
use EasyContent\WordPress\Source\UseCase\PullPost\Handler as PullPostHandler;
use EasyContent\WordPress\Source\UseCase\SetNewStatus\Command;
use EasyContent\WordPress\Source\UseCase\SetNewStatus\Handler;

final class Post
{

    /** @var null|\WP_Post $_wpPostObject Original WordPress post object */
    protected $_wpPostObject;

    /** @var null|integer $_synchronizationTime */
    protected $_synchronizationTime = false;

    /** @var null|ArticleDbModel $_dbArticleObject */
    protected $_dbArticleObject = false;


    /**
     * @param int|\WP_Post $post
     * @throws \Exception
     */
    public function __construct($post)
    {
        $postObject = $post instanceof \WP_Post ? $post : null;

        if ( is_int( $post ) ) {
            if ( $post <= 0 ) {
                throw new \Exception( __( 'Invalid post ID', EASYCONTENT_TXTDOMAIN ), 1 );
            }

            $postObject = get_post( $post );
        }

        if ( null === $postObject ) {
            throw new \Exception( __( 'Error during post fetching', EASYCONTENT_TXTDOMAIN ), 2 );
        }

        $validTypes = ['post', 'page'];

        if ( ! in_array( $postObject->post_type, $validTypes ) ) {
            throw new \Exception( __( 'Invalid post type', EASYCONTENT_TXTDOMAIN ), 3 );
        }

        $this->_wpPostObject = $postObject;
    }


    public function getId()
    {
        return $this->_wpPostObject->ID;
    }


    /**
     * Gets article object based on cached DB data
     *
     * @return ArticleDbModel|null
     */
    public function getArticle()
    {
        if ( false === $this->_dbArticleObject ) {
            $this->loadArticle();
        }

        return $this->_dbArticleObject;
    }


    protected function loadArticle()
    {
        $this->_dbArticleObject = ArticleDbModel::getByPostId( $this->_wpPostObject->ID );
    }


    /**
     * @return int|null Returns UNIX timestamp or NULL if nothing set
     */
    public function getSynchronizationTime()
    {
        if ( false === $this->_synchronizationTime ) {
            $this->_synchronizationTime = $this->isSynced()
                ? ArticlePost::getSynchronizationTime( $this->getArticle()->id, $this->_wpPostObject->ID )
                : null;
        }

        return $this->_synchronizationTime;
    }


    public function getSynchronizationTimeString()
    {
        $time = $this->getSynchronizationTime();

        if ( null === $time ) {
            return ''; // &mdash;
        }

        return TimeHelper::getArticleListTableDateString( $time );
    }


    public function isSynced()
    {
        return (bool)$this->getArticle();
    }

    public function getFrontendData()
    {
        if ( ! $this->isSynced() ) {
            return null;
        }

        $article = $this->getArticle();
        $currentWorkflowStage = $article->getCurrentWorkflowStage();

        return [
            'id'        => $article->id, // Remote article ID. Used for endpoints generation
            'isSynced'  => true,
            'title'     => $article->title,
            'topicName' => $article->topic_name,
            'url'       => $article->getEditRemoteArticleUrl(),
            'content'   => $this->_wpPostObject->post_content, // This probably doesn't work. TODO: Fix this and related quickfixes
            'stage'     => $currentWorkflowStage ? [
                'id'    => $currentWorkflowStage->id,
                'name'  => $currentWorkflowStage->name,
                'color' => $currentWorkflowStage->color
            ] : null,
            'lastUpdatedString' => $this->getSynchronizationTimeString()
        ];
    }

    public function updateMeta(array $params)
    {
        $seoPlugin = SeoPlugin::fromPluginSettings();

        if ($seoPlugin && isset($params['meta_title'])) {
            $seoPlugin->saveMetaTitle($this->_wpPostObject->ID, $params['meta_title']);
        }

        if ($seoPlugin && isset($params['meta_description'])) {
            $seoPlugin->saveMetaDescription($this->_wpPostObject->ID, $params['meta_description']);
        }
    }

    public function setContent($content = '', array $dataArticle = [])
    {
        $postContent = $content
            ? ( new PostContentProcessor( $content, $this->_wpPostObject->ID ) )->run()
            : '';

        $result = wp_update_post( [
            'ID' => $this->_wpPostObject->ID,
            'post_content' => $postContent,
        ], true );

        if ( ! is_wp_error( $result ) ) {
            $this->_wpPostObject->post_content = $postContent;
        }

        return ! is_wp_error( $result );
    }

    public static function onDelete($postId)
    {
        try {
            $post = new Post( (int)$postId ); // Used to validate postId and other things only
            ArticlePost::deleteByPrimaryKey( $postId, 'post_id' );
        } catch ( \Exception $e ) {
        }
    }

    public static function onSave($postId, $post, $update)
    {
        try {
            $newWorkflowStageId = isset( $_POST['ec_new_article_ws_id'] ) && isset( $_POST['ec_change_article_ws'] )
                ? absint( $_POST['ec_new_article_ws_id'] )
                : 0;

            if ($newWorkflowStageId > 0) {
                $command = new Command($postId, $newWorkflowStageId);
                /** @var Handler $handler */
                $handler = Plugin::getInstance()->getContainer()->get(Handler::class);

                $handler->handle($command);
            }


            $pullPost = isset( $_POST['ec_pull_post'] );
            $pushPost = isset( $_POST['ec_push_post'] );

            // Unset variables to prevent multiple execution onPostPull
            unset( $_POST['ec_pull_post'], $_POST['ec_push_post'] );

            if ( ! $pullPost && ! $pushPost ) {
                return;
            }

            $container = Plugin::getInstance()->getContainer();

            $command = $pullPost
                ? new PullPostCommand($postId)
                : new PushPostCommand($postId);

            /** @var PullPostHandler|PushPostHandler $handler */
            $handler = $pullPost
                ? $container->get(PullPostHandler::class)
                : $container->get(PushPostHandler::class);

            $handler->handle($command);
        } catch ( \Exception $e ) {
            // TODO: add notice
        }
    }
}
