<?php

namespace EasyContent\WordPress\Source\Entity\Article;

use EasyContent\WordPress\Source\Entity\Post\PostStatus;
use EasyContent\WordPress\Source\Entity\PostTaxonomiesStorage;
use EasyContent\WordPress\Source\Entity\Post\PostType;

final class Extra
{
    /** @var PostType $postType */
    private $postType;
    /** @var PostStatus $postStatus */
    private $postStatus;
    /** @var array|int[] $categories */
    private $categories = [];
    /** @var array|int[] $tags */
    private $tags = [];
    /** @var null|\DateTimeImmutable $date */
    private $date;

    private function __construct() { }

    private function setPostType(array $parameters, PostType $defaultPostType)
    {
        $this->postType = $defaultPostType;

        if (empty($parameters['type'])) {
            return $this;
        }

        try {
            $this->postType = new PostType($parameters['type']);
        } catch (\Exception $e) { }

        return $this;
    }

    private function setPostStatus(array $parameters, PostStatus $defaultPostStatus)
    {
        $this->postStatus = $defaultPostStatus;

        if (empty($parameters['status'])) {
            return $this;
        }

        try {
            $this->postStatus = new PostStatus($parameters['status']);
        } catch (\Exception $e) { }

        return $this;
    }

    private function setCategories(array $parameters, PostTaxonomiesStorage $taxonomies)
    {
        if (empty($parameters['categories'])) {
            return $this;
        }

        $this->categories = $taxonomies->validateCategories($parameters['categories']);

        return $this;
    }

    private function setTags(array $parameters, PostTaxonomiesStorage $taxonomies)
    {
        if (empty($parameters['tags'])) {
            return $this;
        }

        $this->tags = $taxonomies->validateTags($parameters['tags']);

        return $this;
    }

    private function setDate(array $parameters)
    {
        $dateTime = !empty($parameters['date'])
            ? \DateTimeImmutable::createFromFormat('Y-m-d H:i', $parameters['date'])
            : false;

        $dateTime = false === $dateTime ? null : $dateTime;

        if (!$dateTime) {
            return $this;
        }

        $now = new \DateTimeImmutable;
        $interval = $now->diff($dateTime);

        if (0 !== $interval->invert) {
            return $this;
        }

        $this->date = $dateTime;
        $this->postStatus = PostStatus::future();

        return $this;
    }

    public static function fromArray(
        array $parameters,
        PostType $defaultPostType,
        PostStatus $defaultPostStatus,
        PostTaxonomiesStorage $taxonomies
    ) {
        $extra = (new self)
            ->setPostType($parameters, $defaultPostType)
            ->setPostStatus($parameters, $defaultPostStatus)
            ->setDate($parameters);

        if ($extra->getPostType()->isPost()) {
            $extra
                ->setCategories($parameters, $taxonomies)
                ->setTags($parameters, $taxonomies);
        }

        return $extra;
    }

    /**
     * @return PostType
     */
    public function getPostType()
    {
        return $this->postType;
    }

    /**
     * @return PostStatus
     */
    public function getPostStatus()
    {
        return $this->postStatus;
    }

    /**
     * @return array|int[]
     */
    public function getCategories()
    {
        return $this->categories;
    }

    /**
     * @return array|int[]
     */
    public function getTags()
    {
        return $this->tags;
    }

    /**
     * @return \DateTimeImmutable|null
     */
    public function getDate()
    {
        return $this->date;
    }
}
